Common subdirectories: drakma-old/doc and drakma-new/doc
diff -u drakma-old/request.lisp drakma-new/request.lisp
--- drakma-old/request.lisp	2007-06-11 12:48:20.000000000 -0400
+++ drakma-new/request.lisp	2007-06-11 12:54:34.000000000 -0400
@@ -61,27 +61,33 @@
 (defun make-form-data-function (parameters boundary)
   "Creates and returns a closure which can be used as an argument for
 SEND-CONTENT to send PARAMETERS as a `multipart/form-data' request
-body using the boundary BOUNDARY."
+body using the boundary BOUNDARY. 
+If a file is a part of parameters it may be a pathname, a stream or a function 
+which takes one argument, the stream to the server"
   (lambda (stream)
     (flet ((crlf ()
-             "Sends carriage return and linefeed to STREAM."
-             (write-char #\Return stream)
-             (write-char #\Linefeed stream)))
+	     "Sends carriage return and linefeed to STREAM."
+	     (write-char #\Return stream)
+	     (write-char #\Linefeed stream)))
       (dolist (name/value parameters)
-        (destructuring-bind (name . value)
-            name/value
-          (when (pathnamep value)
-            (setq value (list value)))
-          (format stream "--~A" boundary)
-          (crlf)
-          (format stream "Content-Disposition: form-data; name=\"~A\"" name)
-          (cond ((stringp value)
-                 (crlf) (crlf)
-                 (format stream "~A" value))
-                ((listp value)
-                 (let* ((pathname (first value))
-                        (filename (or (getf (rest value) :filename)
-                                      (file-namestring pathname)))
+	(destructuring-bind (name . value)
+	    name/value
+	  (when (or (pathnamep value)
+		    (streamp value)
+		    (functionp value))
+	    (setq value (list value)))
+	  (format stream "--~A" boundary)
+	  (crlf)
+	  (format stream "Content-Disposition: form-data; name=\"~A\"" name)
+	  (cond ((stringp value)
+		 (crlf) (crlf)
+		 (format stream "~A" value))
+		((listp value)
+		 (let* ((file-source (first value))
+                        (filename (or (if (functionp file-source) "user-closure")
+                                      (if (streamp file-source) "user-stream")
+                                      (getf (rest value) :filename)
+                                      (file-namestring file-source)))
                         (content-type (or (getf (rest value) :content-type)
                                           "application/octet-stream")))
                    (format stream "; filename=\"~A\"" filename)
@@ -89,7 +95,7 @@
                    (format stream "Content-Type: ~A" content-type)
                    (crlf) (crlf)
                    ;; use SEND-CONTENT to send file as binary data
-                   (send-content pathname stream)))
+                   (send-content file-source stream)))
                 (t (error "Don't know what to do with ~S in multipart/form-data body." value)))
           (crlf)))
       (format stream "--~A--" boundary)
@@ -201,7 +207,8 @@
 unless FORM-DATA is true in which case the POST request body is
 sent as `multipart/form-data' using EXTERNAL-FORMAT-OUT.  The
 values of the PARAMETERS alist can also be pathnames or lists
-where the first element is a pathname.  These values denote files
+where the first element is a pathname, a stream or a function 
+which takes one argument, the stream to the server.  These values denote files
 which should be sent as part of the request body, i.e. if files
 are present in PARAMETERS, the content type of the request is
 always `multipart/form-data'.  If the value is a list, the part
